CREATE OR REPLACE FUNCTION create_table_function( tbl_date INTEGER )
RETURNS CHAR(30) AS $$
DECLARE
table_name CHAR(30);
_hour CHAR(2);
_starttime integer;
_endtime integer;
BEGIN
	table_name := message_table_name(tbl_date);
	IF EXISTS (
		SELECT *
		FROM   pg_catalog.pg_tables 
		WHERE    tablename  = table_name
		) THEN
		RETURN table_name;
	ELSE
		_hour := (select extract(hour from (select TIMESTAMP 'epoch' at time zone 'UTC' + tbl_date * INTERVAL '1 second')::timestamp))::text;
		_starttime := (select EXTRACT(EPOCH FROM (select (select TIMESTAMP 'epoch' at time zone 'UTC' + tbl_date * INTERVAL '1 second')::date::text || ' ' || _hour || ':00' )::timestamp));
		_endtime := (select _starttime + 3600);
		EXECUTE ( 'CREATE TABLE '
			|| quote_ident(table_name)
			|| ' ( CHECK ( rtime < '
			|| _endtime
			|| ' AND rtime >= '
			|| _starttime
			|| '), PRIMARY KEY (id, rtime)
		) INHERITS (messages);' );
	END IF;
	RETURN table_name;
END;
$$
LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION messages_insert_trigger()
RETURNS TRIGGER AS $$
DECLARE
table_name CHAR(30);
BEGIN
	table_name := create_table_function(NEW.rtime);
	EXECUTE 'INSERT INTO ' ||  quote_ident(table_name) || ' (ip, rtime, facility, severity, mtime, host, ident, pid, event, msg, fts) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11);'
		USING NEW.ip, NEW.rtime, NEW.facility, NEW.severity, NEW.mtime, NEW.host, NEW.ident, NEW.pid, NEW.event, NEW.msg, NEW.fts;
	PERFORM partition_maintenance_function();
	RETURN NULL;
END;
$$
LANGUAGE plpgsql;
UPDATE service SET version = 28;
