CREATE OR REPLACE FUNCTION messages_insert_trigger()
RETURNS TRIGGER AS $$
DECLARE
table_name CHAR(30);
BEGIN
	table_name := create_table_function(NEW.rtime);
	BEGIN
		EXECUTE 'INSERT INTO ' ||  quote_ident(table_name) || ' (ip, rtime, facility, severity, mtime, host, ident, pid, event, msg, fts) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11);'
		    USING NEW.ip, NEW.rtime, NEW.facility, NEW.severity, NEW.mtime, NEW.host, NEW.ident, NEW.pid, NEW.event, convert_from(convert_to(NEW.msg, 'UTF8'), 'UTF8'), NEW.fts;
	EXCEPTION WHEN SQLSTATE '22P05' OR SQLSTATE '22021' THEN
		BEGIN
			-- We assume that NEW.msg is UTF8 and verify that by converting from UTF8 to UTF8 again.
			-- If this fails we get SQLSTATE 22021, thus we know that it's another encoding and the only other encoding that we support is Windows 1252.
			EXECUTE 'INSERT INTO ' ||  quote_ident(table_name) || ' (ip, rtime, facility, severity, mtime, host, ident, pid, event, msg, fts) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11);'
			    USING NEW.ip, NEW.rtime, NEW.facility, NEW.severity, NEW.mtime, NEW.host, NEW.ident, NEW.pid, NEW.event, convert_from(convert(convert_to(NEW.msg, 'WIN1252'), 'WIN1252', 'UTF8'), 'UTF8'), NEW.fts;
		EXCEPTION WHEN SQLSTATE '22P05' OR SQLSTATE '22021' THEN
			EXECUTE 'INSERT INTO ' ||  quote_ident(table_name) || ' (ip, rtime, facility, severity, mtime, host, ident, pid, event, msg, fts) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11);'
			    USING NEW.ip, NEW.rtime, NEW.facility, NEW.severity, NEW.mtime, NEW.host, NEW.ident, NEW.pid, NEW.event, 'Conversion error: Expected UTF8 or Windows 1252', NEW.fts;
		END;
	END;
	PERFORM partition_maintenance_function();
	RETURN NULL;
END;
$$
LANGUAGE plpgsql;

UPDATE service SET version = 37;
